﻿using Syncfusion.SfChart.XForms;
using Syncfusion.SfChart.XForms.WinPhone;
using Syncfusion.UI.Xaml.Charts;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;
using System.Windows.Media;
using Xamarin.Forms;

[assembly: ExportRenderer(typeof(CustomLabel.CustomExt), typeof(CustomLabel.WinPhone.CustomRenderer))]
namespace CustomLabel.WinPhone
{
    class CustomRenderer : SfChartRenderer
    {
        internal static CustomExt Chart {get; set; }

        protected override void OnElementChanged(Xamarin.Forms.Platform.WinPhone.ElementChangedEventArgs<Syncfusion.SfChart.XForms.SfChart> e)
        {
            base.OnElementChanged(e);
            Chart = e.NewElement as CustomExt;
            ChartLabelStyle labelStyle = Chart.Series[0].DataMarker.LabelStyle;
            StringBuilder sb = new StringBuilder();
            Chart = e.NewElement as CustomExt;

            var textBoxString = "";
            var borderString = "";

            SetDataTemplate(ref borderString, ref textBoxString, labelStyle.BackgroundColor,
                 labelStyle.BorderColor, labelStyle.BorderThickness,
                 labelStyle.TextColor, labelStyle.Font, labelStyle.Margin);
            sb.Clear();
            sb.Append("<DataTemplate xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"");
            sb.Append(" xmlns:x=\"http://schemas.microsoft.com/winfx/2006/xaml\"");
            sb.Append(" xmlns:valueConverter='clr-namespace:CustomLabel.WinPhone;assembly=CustomLabel.WinPhone'>");
            sb.Append("<Border " + borderString + " >");
            sb.Append("<Border.Resources>");
            sb.Append("<valueConverter:LabelConverter x:Key='labelConverter'/>");
            sb.Append("</Border.Resources>");
            sb.Append("<TextBlock Text='{Binding Converter={StaticResource labelConverter}, ConverterParameter={StaticResource labelConverter}}\' " + textBoxString + "/>");
            //sb.Append("<TextBlock "+ textBoxString + "/>");
            sb.Append("</Border></DataTemplate>");

            foreach( var series in Control.Series.ToArray()){
            (series as Syncfusion.UI.Xaml.Charts.LineSeries).AdornmentsInfo.SegmentLabelContent = Syncfusion.UI.Xaml.Charts.LabelContent.LabelContentPath;
            }

            System.Windows.DataTemplate template = (System.Windows.DataTemplate)XamlReader.Load(sb.ToString());
            Border border = template.LoadContent() as Border;
            TextBlock text  = border.Child as TextBlock;
            var binding = text.GetBindingExpression(TextBlock.TextProperty);
           // (binding.ParentBinding.ConverterParameter as LabelConverter).Chart = Chart;

            foreach (var series in Control.Series.ToArray())
            {
                (series as Syncfusion.UI.Xaml.Charts.LineSeries).AdornmentsInfo.LabelTemplate = template;
            }
        }

        internal static void SetDataTemplate(ref string borderString, ref string textBoxString, Xamarin.Forms.Color backgroundColor, Xamarin.Forms.Color borderColor, Xamarin.Forms.Thickness borderThickness, Xamarin.Forms.Color textColor, Xamarin.Forms.Font textFont, Xamarin.Forms.Thickness textMargin)
        {
            if (backgroundColor != Xamarin.Forms.Color.Default)
            {
                borderString += " Background=\"" + new SolidColorBrush(SfChartRenderer.GetWindowsColor(backgroundColor)).Color.ToString() + "\"";
            }
            else
            {
                borderString += " Background='{Binding Interior}'";
            }
            if (borderColor != Xamarin.Forms.Color.Default)
            {
                borderString += " BorderBrush=\"" + new SolidColorBrush(SfChartRenderer.GetWindowsColor(borderColor)).Color.ToString() + "\"";
            }
            borderString += " BorderThickness=\"" + borderThickness.Top + " " + borderThickness.Right + " " + borderThickness.Bottom + " " + borderThickness.Left + "\"";

            if (textColor != Xamarin.Forms.Color.Default)
            {
                textBoxString += " Foreground=\"" + new SolidColorBrush(SfChartRenderer.GetWindowsColor(textColor)).Color.ToString() + "\"";
            }
            if (textFont.FontSize != 0)
            {
                textBoxString += " Fontsize =\"" + textFont.FontSize.ToString() + "\"";
            }
            if (textFont.FontFamily != null)
            {
                textBoxString += " FontFamily =\"" + textFont.FontFamily + "\"";
            }

            if (textFont.FontAttributes == Xamarin.Forms.FontAttributes.Bold)
                textBoxString += " FontStyle =\"Oblique\"";
            else if (textFont.FontAttributes == Xamarin.Forms.FontAttributes.None)
                textBoxString += " FontStyle =\"Normal\"";
            else
                textBoxString += " FontStyle =\"" + textFont.FontAttributes + "\"";

            textBoxString += " Margin =\"" + textMargin.Top + " " + textMargin.Right + " " + textMargin.Bottom + " " + textMargin.Left + "\"";
        }
    }

    public class LabelConverter : DependencyObject, System.Windows.Data.IValueConverter
    {
   

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            ChartAdornment adornment = value as ChartAdornment;

            var args = new DataMarkerLabelEventArgs(adornment.Series.Adornments.IndexOf(adornment), adornment.YData.ToString());
            CustomRenderer.Chart.TriggerDataMarkerLabelCreatedEvent(args);
            return args.LabelContent;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return value;
        }
    }
}
